@@ -15,8 +15,8 @@ def main():
     """Main function."""
     args = parse_args()
     raw_artists = scan_wrapper(args.directory, args.cache)
-    concerts_list = bit(raw_artists, args)
-    output_wrapper(concerts_list, args.output)
+    concert_list = bit(raw_artists, args)
+    output_wrapper(concert_list, args.output)
 
 
 if __name__ == "__main__":
