@@ -11,24 +11,23 @@ import pickle
 import mutagen
 
 
-def has_changed(song_cache, fullpath):
+def has_changed(song_cache, new_song_cache, fullpath):
     """Query song cache to know if file has changed (or is new) since last run.
-    If that is the case, add it to the cache"""
-    # TODO: files that have been deleted are not currently handled
-    file_has_changed = False
+    In any case, update the cache"""
+    file_changed = False
     mtime = os.stat(fullpath).st_mtime
     if (fullpath not in song_cache) or (fullpath in song_cache and
                                         song_cache[fullpath] != mtime):
-        file_has_changed = True
-        song_cache[fullpath] = mtime
-    return song_cache, file_has_changed
+        file_changed = True
+    new_song_cache[fullpath] = mtime
+    return new_song_cache, file_changed
 
 
-def write_song_cache(song_cache, cache_dir):
+def write_song_cache(data, cache_dir):
     """Write song cache file to disk"""
     song_cache_file = os.path.join(cache_dir, 'song_cache')
     with open(song_cache_file, 'wb') as _cache:
-        pickle.dump(song_cache, _cache)
+        pickle.dump(data, _cache)
 
 
 def get_song_cache(cache_dir):
@@ -41,11 +40,11 @@ def get_song_cache(cache_dir):
     return song_cache
 
 
-def write_artist_cache(raw_artists, cache_dir):
+def write_artist_cache(data, cache_dir):
     """Write artist cache file to disk"""
     artist_cache_file = os.path.join(cache_dir, 'artist_cache')
     with open(artist_cache_file, 'w+', encoding='utf-8') as _cache:
-        _cache.write('\n'.join(raw_artists))
+        _cache.write('\n'.join(data))
 
 
 def get_artist_cache(cache_dir):
@@ -58,38 +57,52 @@ def get_artist_cache(cache_dir):
     return raw_artists
 
 
-def add_artist(fullpath, raw_artists):
-    """Add artist to artist list"""
-    # TODO: fix this try-except? IIRC, mutagen crashed on my
-    # full music collection and I didn't wanted to debug it at
-    # the time.
+def modify_artist(fullpath, raw_artists, action):
+    """Modify artist list"""
     try:
-        artist = mutagen.File(fullpath)["artist"]
+        artist = mutagen.File(fullpath, easy=True)["artist"]
         artist = ''.join(artist)  # list to string
         split_pattern = re.compile(r"\+|/|&")
         if re.search(split_pattern, artist):
             splitted_artist = re.split(split_pattern, artist)
             for artist in splitted_artist:
-                raw_artists.append(artist.lstrip())
+                if action == 'add':
+                    raw_artists.append(artist.lstrip())
+                elif action == 'remove':
+                    raw_artists.remove(artist.lstrip())
         else:
-            raw_artists.append(artist)
-    except:  # pylint: disable=W0702
+            if action == 'add':
+                raw_artists.append(artist)
+            elif action == 'remove':
+                raw_artists.remove(artist)
+    except KeyError:  # no artist tag
+        pass
+    except mutagen.flac.FLACNoHeaderError:  # FLAC file has no header
         pass
     return raw_artists
 
 
-def scan_dir(music_dir, cache_dir):
+def scan_dir(music_dir, song_cache, raw_artists):
     """Scan a directory and output a list of artists"""
-    raw_artists = get_artist_cache(cache_dir)
-    song_cache = get_song_cache(cache_dir)
+    new_song_cache = {}
     for dirname, _, filenames in os.walk(music_dir, topdown=False):
         for song in filenames:
             fullpath = os.path.abspath(os.path.join(dirname, song))
             if song.endswith(('.flac', '.mp3', '.ogg')):
-                song_cache, file_has_changed = has_changed(song_cache, fullpath)
-                if file_has_changed:
-                    raw_artists = add_artist(fullpath, raw_artists)
-    raw_artists = set(raw_artists)
-    write_song_cache(song_cache, cache_dir)
+                new_song_cache, file_changed = has_changed(song_cache, new_song_cache, fullpath)
+                if file_changed:
+                    raw_artists = modify_artist(fullpath, raw_artists, 'add')
+    stale_song_cache = set(song_cache) - set(new_song_cache)
+    for song in stale_song_cache:
+        raw_artists = modify_artist(song, raw_artists, 'remove')
+    return set(raw_artists), new_song_cache
+
+
+def scan_wrapper(music_dir, cache_dir):
+    """Wrapper function to manage the scan"""
+    raw_artists = get_artist_cache(cache_dir)
+    song_cache = get_song_cache(cache_dir)
+    raw_artists, new_song_cache = scan_dir(music_dir, song_cache, raw_artists)
+    write_song_cache(new_song_cache, cache_dir)
     write_artist_cache(raw_artists, cache_dir)
     return raw_artists
